@@ -1,10 +1,32 @@
+import tempfile
+import time
+
 import pytest
-from requests import HTTPError
 
 from garth.auth_tokens import OAuth2Token
+from garth.exc import GarthHTTPError
 from garth.http import Client
 
 
+def test_dump_and_load(authed_client: Client):
+    with tempfile.TemporaryDirectory() as tempdir:
+        authed_client.dump(tempdir)
+
+        new_client = Client()
+        new_client.load(tempdir)
+
+        assert new_client.oauth1_token == authed_client.oauth1_token
+        assert new_client.oauth2_token == authed_client.oauth2_token
+
+
+def test_dumps_and_loads(authed_client: Client):
+    s = authed_client.dumps()
+    new_client = Client()
+    new_client.loads(s)
+    assert new_client.oauth1_token == authed_client.oauth1_token
+    assert new_client.oauth2_token == authed_client.oauth2_token
+
+
 def test_configure_oauth2_token(client: Client, oauth2_token: OAuth2Token):
     assert client.oauth2_token is None
     client.configure(oauth2_token=oauth2_token)
@@ -73,8 +95,9 @@ def test_client_request(client: Client):
     resp = client.request("GET", "connect", "/")
     assert resp.ok
 
-    with pytest.raises(HTTPError):
+    with pytest.raises(GarthHTTPError) as e:
         client.request("GET", "connectapi", "/")
+    assert "404" in str(e.value)
 
 
 @pytest.mark.vcr
@@ -93,9 +116,9 @@ def test_login_success_mfa(monkeypatch, client: Client):
 
 @pytest.mark.vcr
 def test_username(authed_client: Client):
-    assert authed_client._username is None
+    assert authed_client._profile is None
     assert authed_client.username
-    assert authed_client._username == authed_client.username
+    assert authed_client._profile
 
 
 @pytest.mark.vcr
@@ -103,6 +126,8 @@ def test_connectapi(authed_client: Client):
     stress = authed_client.connectapi(
         "/usersummary-service/stats/stress/daily/2023-07-21/2023-07-21"
     )
+    assert stress
+    assert isinstance(stress, list)
     assert len(stress) == 1
     assert stress[0]["calendarDate"] == "2023-07-21"
     assert list(stress[0]["values"].keys()) == [
@@ -112,3 +137,32 @@ def test_connectapi(authed_client: Client):
         "restStressDuration",
         "mediumStressDuration",
     ]
+
+
+@pytest.mark.vcr
+def test_refresh_oauth2_token(authed_client: Client):
+    assert authed_client.oauth2_token
+    authed_client.oauth2_token.expires_at = int(time.time())
+    assert authed_client.oauth2_token.expired
+    profile = authed_client.connectapi("/userprofile-service/socialProfile")
+    assert profile
+    assert isinstance(profile, dict)
+    assert profile["userName"]
+
+
+@pytest.mark.vcr
+def test_download(authed_client: Client):
+    downloaded = authed_client.download(
+        "/download-service/files/activity/11998957007"
+    )
+    assert downloaded
+    zip_magic_number = b"\x50\x4B\x03\x04"
+    assert downloaded[:4] == zip_magic_number
+
+
+@pytest.mark.vcr
+def test_upload(authed_client: Client):
+    fpath = "tests/12129115726_ACTIVITY.fit"
+    with open(fpath, "rb") as f:
+        uploaded = authed_client.upload(f)
+    assert uploaded
